@@ -1,7 +1,6 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2013-2018  Brazil
-  Copyright(C) 2018-2020  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -81,13 +80,24 @@ typedef grn_bool (*grn_scan_info_each_arg_callback)(grn_ctx *ctx, grn_obj *obj,
 
 void grn_expr_init_from_env(void);
 
+grn_obj *
+grn_expr_get_query_options(grn_ctx *ctx,
+                           grn_obj *expr);
+grn_rc
+grn_expr_set_query_options(grn_ctx *ctx,
+                           grn_obj *expr,
+                           grn_obj *query_options);
+
 scan_info **grn_scan_info_build(grn_ctx *ctx, grn_obj *expr, int *n,
                                 grn_operator op, grn_bool record_exist);
 
 scan_info *grn_scan_info_open(grn_ctx *ctx, int start);
 void grn_scan_info_close(grn_ctx *ctx, scan_info *si);
-void grn_scan_info_put_index(grn_ctx *ctx, scan_info *si, grn_obj *index,
-                             uint32_t sid, int32_t weight,
+void grn_scan_info_put_index(grn_ctx *ctx,
+                             scan_info *si,
+                             grn_obj *index,
+                             uint32_t sid,
+                             float weight,
                              grn_obj *scorer,
                              grn_obj *scorer_args_expr,
                              uint32_t scorer_args_expr_offset);
@@ -116,7 +126,7 @@ int grn_scan_info_get_start_position(scan_info *si);
 void grn_scan_info_set_start_position(scan_info *si, int start);
 void grn_scan_info_reset_position(scan_info *si);
 
-int32_t grn_expr_code_get_weight(grn_ctx *ctx, grn_expr_code *ec, uint32_t *offset);
+float grn_expr_code_get_weight(grn_ctx *ctx, grn_expr_code *ec, uint32_t *offset);
 grn_rc grn_expr_code_inspect_indented(grn_ctx *ctx,
                                       grn_obj *buffer,
                                       grn_expr_code *code,
@@ -145,11 +155,9 @@ grn_rc grn_expr_module_list_get_arguments(grn_ctx *ctx,
                                           grn_obj *arguments);
 
 grn_rc
-grn_table_select_sequential(grn_ctx *ctx,
-                            grn_obj *table,
-                            grn_obj *expr,
-                            grn_obj *res,
-                            grn_operator op);
+grn_expr_match_columns_split(grn_ctx *ctx,
+                             grn_obj *expr,
+                             grn_obj *splitted_match_columns);
 
 grn_obj *
 grn_expr_slice(grn_ctx *ctx,
@@ -166,10 +174,17 @@ grn_expr_get_range_info(grn_ctx *ctx,
                         grn_id *range_id,
                         grn_obj_flags *range_flags);
 
+typedef enum {
+  GRN_EXPR_V1_FORMAT_TYPE_OUTPUT_COLUMNS,
+  GRN_EXPR_V1_FORMAT_TYPE_SORT_KEYS,
+  GRN_EXPR_V1_FORMAT_TYPE_GROUP_KEYS,
+} grn_expr_v1_format_type;
+
 bool
-grn_expr_is_v1_columns_format(grn_ctx *ctx,
-                              const char *raw_text,
-                              ssize_t raw_text_len);
+grn_expr_is_v1_format(grn_ctx *ctx,
+                      const char *raw_text,
+                      ssize_t raw_text_len,
+                      grn_expr_v1_format_type type);
 
 
 #ifdef __cplusplus
