@@ -1,7 +1,6 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2009-2018  Brazil
-  Copyright(C) 2018-2020  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -20,8 +19,8 @@
 #pragma once
 
 #include "grn.h"
-#include "grn_ctx.h"
 #include "grn_ii_select_cursor.h"
+#include "grn_io.h"
 #include "grn_table_module.h"
 
 #ifdef __cplusplus
