@@ -1,4 +1,3 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2013-2018  Brazil
   Copyright(C) 2019-2020  Kouhei Sutou <kou@clear-code.com>
@@ -107,7 +106,7 @@ mrb_grn_scan_info_put_index(mrb_state *mrb, mrb_value self)
   scan_info *si;
   mrb_value mrb_index;
   mrb_int sid;
-  mrb_int weight;
+  mrb_float weight;
   mrb_value mrb_scorer;
   mrb_value mrb_scorer_args_expr;
   mrb_int scorer_args_expr_offset;
@@ -115,7 +114,7 @@ mrb_grn_scan_info_put_index(mrb_state *mrb, mrb_value self)
   grn_obj *scorer = NULL;
   grn_obj *scorer_args_expr = NULL;
 
-  mrb_get_args(mrb, "oiiooi",
+  mrb_get_args(mrb, "oifooi",
                &mrb_index, &sid, &weight,
                &mrb_scorer,
                &mrb_scorer_args_expr,
@@ -410,7 +409,7 @@ mrb_grn_expr_code_inspect(mrb_state *mrb, mrb_value self)
   mrb_str_concat(mrb, inspected, mrb_ptr_to_str(mrb, mrb_cptr(self)));
 
   {
-    int32_t weight;
+    float weight;
     uint32_t offset;
 
     weight = grn_expr_code_get_weight(ctx, DATA_PTR(self), &offset);
@@ -418,7 +417,7 @@ mrb_grn_expr_code_inspect(mrb_state *mrb, mrb_value self)
     mrb_str_cat_lit(mrb, inspected, " weight=");
     mrb_str_concat(mrb, inspected,
                    mrb_funcall(mrb,
-                               mrb_int_value(mrb, weight),
+                               mrb_float_value(mrb, weight),
                                "inspect",
                                0));
     mrb_str_cat_lit(mrb, inspected, ", offset=");
@@ -473,12 +472,12 @@ static mrb_value
 mrb_grn_expr_code_get_weight(mrb_state *mrb, mrb_value self)
 {
   grn_ctx *ctx = (grn_ctx *)mrb->ud;
-  int32_t weight;
+  float weight;
   uint32_t offset;
   mrb_value mrb_values[2];
 
   weight = grn_expr_code_get_weight(ctx, DATA_PTR(self), &offset);
-  mrb_values[0] = mrb_int_value(mrb, weight);
+  mrb_values[0] = mrb_float_value(mrb, weight);
   mrb_values[1] = mrb_int_value(mrb, offset);
   return mrb_ary_new_from_values(mrb, 2, mrb_values);
 }
@@ -804,7 +803,11 @@ mrb_grn_expression_append_constant(mrb_state *mrb, mrb_value self)
     }
     break;
   case MRB_TT_INTEGER:
-    grn_expr_append_const_int(ctx, expr, mrb_integer(mrb_constant), op, n_args);
+    grn_expr_append_const_int32(ctx,
+                                expr,
+                                mrb_integer(mrb_constant),
+                                op,
+                                n_args);
     break;
   case MRB_TT_SYMBOL :
     {
@@ -817,11 +820,20 @@ mrb_grn_expression_append_constant(mrb_state *mrb, mrb_value self)
     break;
   case MRB_TT_FLOAT :
     {
-      grn_obj constant;
-      GRN_FLOAT_INIT(&constant, 0);
-      GRN_FLOAT_SET(ctx, &constant, mrb_float(mrb_constant));
-      grn_expr_append_const(ctx, expr, &constant, op, n_args);
-      GRN_OBJ_FIN(ctx, &constant);
+      mrb_float value = mrb_float(mrb_constant);
+      if (sizeof(mrb_float) == sizeof(float)) {
+        grn_expr_append_const_float32(ctx,
+                                      expr,
+                                      value,
+                                      op,
+                                      n_args);
+      } else {
+        grn_expr_append_const_float(ctx,
+                                    expr,
+                                    value,
+                                    op,
+                                    n_args);
+      }
     }
     break;
   case MRB_TT_STRING :
