@@ -1,7 +1,6 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2017-2018 Brazil
-  Copyright(C) 2018-2020 Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2017-2018  Brazil
+  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -19,6 +18,7 @@
 
 #include "grn.h"
 #include "grn_ctx.h"
+#include "grn_ctx_impl.h"
 #include "grn_expr_executor.h"
 
 grn_rc
@@ -641,3 +641,48 @@ exit :
   }
   GRN_API_RETURN(have_duplicated_keys);
 }
+
+grn_hash *
+grn_table_all_columns(grn_ctx *ctx, grn_obj *table)
+{
+  if (!GRN_OBJ_TABLEP(table)) {
+    return NULL;
+  }
+
+  grn_id id = DB_OBJ(table)->id;
+  if (id == GRN_ID_NIL) {
+    return NULL;
+  }
+
+  CRITICAL_SECTION_ENTER(ctx->impl->columns_cache_lock);
+  void *value;
+  grn_hash *columns = NULL;
+  grn_id cache_id = grn_hash_get(ctx,
+                                 ctx->impl->columns_cache,
+                                 &id,
+                                 sizeof(grn_id),
+                                 &value);
+  if (cache_id != GRN_ID_NIL) {
+    columns = *((grn_hash **)value);
+  } else {
+    int added = 0;
+    cache_id = grn_hash_add(ctx,
+                            ctx->impl->columns_cache,
+                            &id,
+                            sizeof(grn_id),
+                            &value,
+                            &added);
+    if (cache_id != GRN_ID_NIL && added) {
+      columns = grn_hash_create(ctx,
+                                NULL,
+                                sizeof(grn_id),
+                                0,
+                                GRN_OBJ_TABLE_HASH_KEY|GRN_HASH_TINY);
+      grn_table_columns(ctx, table, "", 0, (grn_obj *)columns);
+      *((grn_hash **)value) = columns;
+    }
+  }
+  CRITICAL_SECTION_LEAVE(ctx->impl->columns_cache_lock);
+
+  return columns;
+}
