@@ -1,4 +1,3 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2010-2018  Brazil
   Copyright(C) 2019-2020  Sutou Kouhei <kou@clear-code.com>
@@ -1118,6 +1117,14 @@ grn_json_load_open_bracket_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
       GRN_TEXT_PUTS(ctx, buf, ", ");
     }
     grn_inspect(ctx, buf, value);
+    switch (value->header.domain) {
+    case GRN_JSON_LOAD_OPEN_BRACKET :
+    case GRN_JSON_LOAD_OPEN_BRACE :
+      i += GRN_UINT32_VALUE(value);
+      break;
+    default :
+      break;
+    }
   }
   GRN_TEXT_PUTS(ctx, buf, "]");
 
@@ -1142,6 +1149,14 @@ grn_json_load_open_brace_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *obj)
     grn_inspect(ctx, buf, key);
     GRN_TEXT_PUTS(ctx, buf, ": ");
     grn_inspect(ctx, buf, value);
+    switch (value->header.domain) {
+    case GRN_JSON_LOAD_OPEN_BRACKET :
+    case GRN_JSON_LOAD_OPEN_BRACE :
+      i += GRN_UINT32_VALUE(value);
+      break;
+    default :
+      break;
+    }
   }
   GRN_TEXT_PUTS(ctx, buf, "}");
 
@@ -1330,7 +1345,7 @@ grn_inspect(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
       grn_json_load_open_brace_inspect(ctx, buffer, obj);
       return buffer;
     default :
-      if (grn_obj_is_table(ctx, grn_ctx_at(ctx, obj->header.domain))) {
+      if (grn_id_maybe_table(ctx, obj->header.domain)) {
         grn_record_inspect(ctx, buffer, obj);
         return buffer;
       }
@@ -1476,7 +1491,11 @@ grn_inspect_limited(grn_ctx *ctx, grn_obj *buffer, grn_obj *obj)
 }
 
 grn_obj *
-grn_inspect_key(grn_ctx *ctx, grn_obj *buffer, grn_obj *table, const void *key, unsigned int key_size)
+grn_inspect_key(grn_ctx *ctx,
+                grn_obj *buffer,
+                grn_obj *table,
+                const void *key,
+                uint32_t key_size)
 {
   if (!buffer) {
     buffer = grn_obj_open(ctx, GRN_BULK, 0, GRN_DB_TEXT);
