@@ -33,10 +33,10 @@ module LaunchpadHelper
 
   def ubuntu_targets_default
     [
-      ["xenial", "16.04"],
       ["bionic", "18.04"],
       ["focal", "20.04"],
       ["groovy", "20.10"],
+      ["hirsute", "21.04"],
     ]
   end
 
